//
//  SingleFriendEmbededViewController.swift
//  KakaoPartnerSDKSample
//
//  Created by kakao on 2/11/25.
//  Copyright © 2025 apiteam. All rights reserved.
//

import UIKit

import KakaoSDKCommon
import KakaoSDKAuth
import KakaoPartnerSDKFriendDelegate

import KakaoSDKTalk
import KakaoPartnerSDKTalk

class SingleFriendEmbededViewController: BaseEmbededViewController {
    override func viewDidLoad() {
        getPickerView()
    }
    
    override func updateViewConstraints() {
        super.updateViewConstraints()
    }
    
    func getPickerView() {
        TalkApi.shared.friendsForPartner(context: PartnerFriendsContext()) { [weak self] friends, error in
            if let error {
                self?.showAlert(value: error)
                return
            }
            
            if let friends, let users = friends.elements {
                self?.getPickerViewWithCustomSection(users: users)
            }
        }
    }
    
    private func getPickerViewWithCustomSection(users: [PartnerFriend]) {
        guard let appKey = try? KakaoSDK.shared.appKey(),
              let bundleId = Bundle.main.bundleIdentifier,
              let accessToken = TokenManager.manager.getToken()?.accessToken else {
            dismiss(animated: true)
            
            return
        }
                
        let params = PickerFriendRequestParams(showFavorite: false)
        let targetInfo = TargetInfo(appKey: appKey, bundleId: bundleId, accessToken: accessToken)
        let uuids = users.map { $0.uuid }
        
        PickerApi.shared.singleFriendPickerView(
            params: params,
            targetInfo: targetInfo,
            selectedUuids: uuids,
            customSection: CustomSection(title: "Custom Section in single", uuids: uuids)
        ) { [weak self] resultView, error in
            if let error {
                print("##### error occur: \(error)")

                DispatchQueue.main.async {
                    self?.dismiss(animated: true)
                }
            }

            if let resultView {
                self?.pickerView = resultView
                self?.setLayout()
                self?.setDelegate()
            }
        }
    }
    
    func setDelegate() {
        guard let pickerView else { return }
        
        pickerView.setSelectedUserCallback(self)
        pickerView.setCustomFavoriteCallback(self)
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: any UIViewControllerTransitionCoordinator) {
        coordinator.animate { [weak self] _ in
            self?.pickerView?.updateSubviewsConstraint()
        }
    }
}

// MARK: - SelectedUserCallback
extension SingleFriendEmbededViewController: SelectedUserCallback {
    func select(user: SelectedUsers) {
        print("#### selected user: \(user.users?.first?.uuid)")
        showAlert(value: user)
    }
}

// MARK: - FavoriteUserDelegate
extension SingleFriendEmbededViewController: CustomFavoriteCallback {
    func selectUser(uuid: String) {
        print("##### favorite user: \(uuid)")
    }
    
    func deselectUser(uuid: String) {
        print("##### not favorite user: \(uuid)")
    }
}

